unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, RVFuncs, DragDrop, DropSource, DragDropInternet,
  DragDropGraphics, ExtCtrls, RVStyle, RVScroll, RichView, RVEdit, StdCtrls,
  ActnList, RichViewActions, RVTypes, RVFileFuncs;

{
 This demo requires TRichView 17.3 or newer.


 This demo shows how to implement drag&drop of Delphi components to TRichViewEdit.
 This demo uses Drag&Drop Component Suite:
 - new version: https://github.com/DelphiPraxis/The-Drag-and-Drop-Component-Suite-for-Delphi
 - original version: http://melander.dk/

 This demo uses RichViewActions, because TrvActionInsertHyperlink contains several
 useful methods (GetHyperlinkStyleNo, TerminateHyperlink, GoToLink). You can
 implement similar methods youself, see Demos\DelphiUnicode\Assorted\Hypertext\URLs\
}

type
  TForm1 = class(TForm)
    TreeView1: TTreeView;
    DropURLSource1: TDropURLSource;
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Image1: TImage;
    DropBMPSource1: TDropBMPSource;
    Label1: TLabel;
    ActionList1: TActionList;
    rvActionInsertHyperlink1: TrvActionInsertHyperlink;
    procedure TreeView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure RichViewEdit1ReadHyperlink(Sender: TCustomRichView;
      const Target, Extras: TRVUnicodeString; DocFormat: TRVLoadFormat;
      var StyleNo: Integer; var ItemTag: TRVTag; var ItemName: TRVUnicodeString);
    procedure RichViewEdit1KeyPress(Sender: TObject; var Key: Char);
    procedure RichViewEdit1Jump(Sender: TObject; id: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  TreeView1.Items[0].Expand(True);
  RichViewEdit1.Clear;
  RichViewEdit1.AddNL('Drop here!', 0, 0);
  RichViewEdit1.Format;
  // rvddURL is not included by default
  RichViewEdit1.AcceptDragDropFormats := RichViewEdit1.AcceptDragDropFormats+[rvddURL];
end;

// Dragging URLs from TreeView
procedure TForm1.TreeView1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var Node: TTreeNode;
begin
  Node := TTreeView(Sender).GetNodeAt(X,Y);
  if (Node=nil) or not RVIsURL(Node.Text) then
    exit;
  if (DragDetectPlus(TWinControl(Sender))) then
  begin
    DropURLSource1.Title := Node.Text;
    DropURLSource1.URL   := Node.Text;
    DropURLSource1.Execute;
  end;
end;
// Dragging bitmap
procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var p: TPoint;
begin
  p := TControl(Sender).ClientToScreen(Point(X,Y));
  if DragDetectPlus(TImage(Sender).Parent, p) then
  begin
    DropBMPSource1.Bitmap.Assign(TImage(Sender).Picture.Bitmap);
    DropBMPSource1.Execute;
  end;
end;
// This even allows defining style for dropped links
procedure TForm1.RichViewEdit1ReadHyperlink(Sender: TCustomRichView;
  const Target, Extras: TRVUnicodeString; DocFormat: TRVLoadFormat;
  var StyleNo: Integer; var ItemTag: TRVTag; var ItemName: TRVUnicodeString);
begin
  if DocFormat=rvlfURL then
    StyleNo :=
      rvActionInsertHyperlink1.GetHyperlinkStyleNo(RichViewEdit1);
  ItemTag := rvActionInsertHyperlink1.EncodeTarget(Target);
end;
// Hyperlink autodetection and termination
procedure TForm1.RichViewEdit1KeyPress(Sender: TObject; var Key: Char);
begin
  if Key in [#13, #9, ' ', ',', ';'] then
  begin
    rvActionInsertHyperlink1.DetectURL(RichViewEdit1);
    rvActionInsertHyperlink1.TerminateHyperlink(RichViewEdit1);
  end;
end;
// On hyperlink click
procedure TForm1.RichViewEdit1Jump(Sender: TObject; id: Integer);
begin
  rvActionInsertHyperlink1.GoToLink(RichViewEdit1, id);
end;

end.
